/* -*- c++ -*- */
/* 
 * Copyright 2019 Thomas Kolb.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_HAMNET70_PID_CONTROLLER_H
#define INCLUDED_HAMNET70_PID_CONTROLLER_H

#include <hamnet70/api.h>
#include <gnuradio/block.h>

namespace gr {
  namespace hamnet70 {

    /*!
     * \brief <+description of block+>
     * \ingroup hamnet70
     *
     */
    class HAMNET70_API pid_controller : virtual public gr::block
    {
     public:
      typedef boost::shared_ptr<pid_controller> sptr;

      /*!
       * \brief Return a shared_ptr to a new instance of hamnet70::pid_controller.
       *
       * To avoid accidental use of raw pointers, hamnet70::pid_controller's
       * constructor is in a private implementation
       * class. hamnet70::pid_controller::make is the public interface for
       * creating new instances.
       */
      static sptr make(size_t interval, float p, float i, float d, float post_gain = 1.0f);
    };

  } // namespace hamnet70
} // namespace gr

#endif /* INCLUDED_HAMNET70_PID_CONTROLLER_H */

